from wog.Live import load_game, welcome


def start_game():
    play_again = True
    valid_input = False
    player_name = input("Hi!\nWhat is your name?\n")
    print(welcome(player_name))
    while play_again:
        play_again = False
        valid_input = False
        load_game()
        while not valid_input:
            user_choice = input("\nDo you want to try again?\nPlease enter \'Yes\' or \'No\'.\n")
            if user_choice == 'Yes':
                valid_input = True
                play_again = True
            elif user_choice == 'No':
                valid_input = True
                play_again = False
            else:
                print("Invalid input.\n")



from wog.Game import Game
from random import randint
import sys
from time import sleep


class MemoryGame(Game):
    def __init__(self, difficulty):
        super().__init__(difficulty)

    def __generate_sequence(self):
        computer_sequence = []
        for i in range(self.difficulty):
            rand_num = randint(1, 101)
            computer_sequence.append(rand_num)
        return computer_sequence

    def __get_list_from_user(self):
        user_sequence = []
        print("\rNow try to remember which numbers appeared.")
        for i in range(self.difficulty):
            user_sequence.append(int(input(f"#{i + 1} number: ")))
        return user_sequence

    def __is_list_equal(self, computer_sequence, user_sequence):
        for i in range(self.difficulty):
            if computer_sequence[i] != user_sequence[i]:
                return False
        return True

    def play(self):
        computer_sequence = self.__generate_sequence()
        for number in computer_sequence:
            sys.stdout.write("\r")
            sys.stdout.write(f"{number} ")
            sleep(0.7)
            sys.stdout.flush()
        sys.stdout.flush()
        user_sequence = self.__get_list_from_user()
        print("\nThe numbers that appeared are:", str(computer_sequence))
        return self.__is_list_equal(computer_sequence, user_sequence)


"""World Of Games

Play some little games, have fun :)
You can execute the game by call package name.

To run the game, write:
MainGame.start_game()
after you imported the package.
"""
from wog import MainGame



from wog.GuessGame import GuessGame
from wog.MemoryGame import MemoryGame
from wog.CurrencyRouletteGame import CurrencyRouletteGame


def welcome(name):
    print("==============================")
    print(" _       __           ___")
    print("| |     / /  ___    //")
    print("| | /| / / //  \\\  //  __")
    print("| |/ |/ / //   // //    //")
    print("|_/  |_/  \\\__//  \\\___//")
    print("==============================")
    output = (f"Hello {name} and welcome to the World of Games (WoG).\n"
              f"Here you can find many cool games to play.\n")
    return output


def load_game():
    valid_option = False
    # I'm using a range like an Enum in favor of more readable code.
    memory_game, guess_game, currency_roulette_game = range(1, 4)
    winner_str = "Awesome, You're right! :)"
    looser_str = "It was close, but wrong. :(\nDon\'t give up, maybe next time!"
    while not valid_option:
        selected_game = input(f"\nPlease choose a game to play:\n"
              f"1. Memory Game - a sequence of numbers will appear for 1 second and you have to guess it back.\n"
              f"2. Guess Game - guess a number and see if you chose like the computer.\n"
              f"3. Currency Roulette - try and guess the value of a random amount of USD in ILS.\n")
        if selected_game.isdigit():
            selected_game = int(selected_game)
        if selected_game == memory_game or selected_game == guess_game or selected_game == currency_roulette_game:
            valid_option = True
        else:
            print("Please enter a valid input.")
    valid_option = False
    while not valid_option:
        difficult_level = int(input("Please choose game difficulty from 1 to 5:\n"))
        if (difficult_level >= 1) and (difficult_level <= 5):
            valid_option = True
            if selected_game == memory_game:
                game = MemoryGame(difficult_level)
            elif selected_game == guess_game:
                game = GuessGame(difficult_level)
            elif selected_game == currency_roulette_game:
                game = CurrencyRouletteGame(difficult_level)
            print(winner_str) if game.play() else print(looser_str)
        else:
            print("Please enter a valid input.")



from abc import ABC, abstractmethod


class Game(ABC):
    @abstractmethod
    def __init__(self, difficulty):
        self.difficulty = difficulty

    @abstractmethod
    def play(self):
        pass



from wog.Game import Game
from random import randint


class GuessGame(Game):
    def __init__(self, difficulty):
        super().__init__(difficulty)
        self.__secret_number = 0

    def __generate_number(self):
        print("\nLet\'s try to guess a number.")
        self.__secret_number = randint(1, self.difficulty)

    def __get_guess_from_user(self):
        user_guess = int(input(f"\nPlease guess a number between 1 to {self.difficulty}.\n"))
        return user_guess

    @staticmethod
    def __compare_results(secret_number, user_guess):
        return True if secret_number == user_guess else False

    def play(self):
        self.__generate_number()
        user_number = self.__get_guess_from_user()
        print(f"\nWe chose the number: {self.__secret_number}, you guessed: {user_number}.")
        return self.__compare_results(self.__secret_number, user_number)



from wog.Game import Game
from currency_converter import CurrencyConverter
from random import randint


class CurrencyRouletteGame(Game):
    def __init__(self, difficulty):
        super().__init__(difficulty)

    @staticmethod
    def __calculate_currency(amount_of_dollars):
        currency_converter = CurrencyConverter()
        exchange_rate = currency_converter.convert(amount_of_dollars, 'USD', 'ILS')
        return exchange_rate

    def __get_money_interval(self, usd_ils_value):
        d = self.difficulty
        t = usd_ils_value
        interval = (t - (5 - d), t + (5 - d))
        return interval

    @staticmethod
    def __get_guess_from_user(amount_of_dollars):
        user_guess = input(f"\nCan you guess what is the exchange rate"
                           f" between {amount_of_dollars} Dollar to Shekel(USD -> ILS)?\n")
        return user_guess

    def play(self):
        rand_num = randint(1, 100)
        user_guess = int(self.__get_guess_from_user(rand_num))
        really_exchange_rate = self.__calculate_currency(rand_num)
        limits_tuple = self.__get_money_interval(really_exchange_rate)
        print(f"\nThe current exchange rate: {self.__calculate_currency(1)}.\n"
              f"{rand_num} x {self.__calculate_currency(1)} = {self.__calculate_currency(rand_num)}")
        if (user_guess >= int(limits_tuple[0])) and (user_guess <= int(limits_tuple[1])):
            return True
        else:
            return False
